@@ -4,9 +4,9 @@
 .. module:: GenomeAnalysis
    :synopsis: GenomeAnalysis implements genome analysis specifics
 .. versionadded:: 3.0.0
-.. versionchanged:: 5.0.0
+.. versionchanged:: 5.4.0
 
-Copyright (c) 2016-2021, Evgeny Zdobnov (ez@ezlab.org)
+Copyright (c) 2016-2022, Evgeny Zdobnov (ez@ezlab.org)
 Licensed under the MIT license. See LICENSE.md file.
 
 """
@@ -14,6 +14,7 @@ from busco.analysis.BuscoAnalysis import BuscoAnalysis
 from busco.analysis.Analysis import NucleotideAnalysis, BLASTAnalysis
 from busco.busco_tools.prodigal import ProdigalRunner
 from busco.busco_tools.metaeuk import MetaeukRunner
+from busco.busco_tools.bbtools import BBToolsRunner
 from busco.busco_tools.augustus import (
     AugustusRunner,
     GFF2GBRunner,
@@ -28,6 +29,7 @@ from abc import ABCMeta, abstractmethod
 from configparser import NoOptionError
 import time
 import os
+import glob
 import pandas as pd
 from collections import defaultdict
 import subprocess
@@ -42,10 +44,14 @@ class GenomeAnalysis(NucleotideAnalysis, BuscoAnalysis, metaclass=ABCMeta):
 
     def __init__(self):
         super().__init__()
+        self.bbtools_runner = None
 
     @abstractmethod
     def run_analysis(self):
         super().run_analysis()
+        if "genome" in self.config.get("busco_run", "mode"):
+            # This avoids the child class euk_tran mode running bbtools
+            self._run_bbtools()
 
     def init_tools(self):
         """
@@ -53,41 +59,23 @@ class GenomeAnalysis(NucleotideAnalysis, BuscoAnalysis, metaclass=ABCMeta):
         :return:
         """
         super().init_tools()
+        if "genome" in self.config.get("busco_run", "mode"):
+            # This avoids the child class euk_tran mode running bbtools
+            self.bbtools_runner = BBToolsRunner()
 
-    # def _run_tarzip_augustus_output(self): # Todo: rewrite using tarfile
-    #     """
-    #     This function tarzips results folder
-    #     """
-    #     # augustus_output/predicted_genes
-    #
-    #     self._p_open(["tar", "-C", "%saugustus_output" % self.main_out,
-    #                   "-zcf", "%saugustus_output/predicted_genes.tar.gz" %
-    #                   self.main_out, "predicted_genes", "--remove-files"],
-    #                  "bash", shell=False)
-    #     # augustus_output/extracted_proteins
-    #     self._p_open(["tar", "-C", "%saugustus_output" % self.main_out,
-    #                   "-zcf", "%saugustus_output/extracted_proteins.tar.gz" %
-    #                   self.main_out, "extracted_proteins", "--remove-files"],
-    #                  "bash", shell=False)
-    #     # augustus_output/gb
-    #     self._p_open(["tar", "-C", "%saugustus_output" % self.main_out,
-    #                   "-zcf", "%saugustus_output/gb.tar.gz" % self.main_out, "gb", "--remove-files"],
-    #                  "bash", shell=False)
-    #     # augustus_output/gffs
-    #     self._p_open(["tar", "-C", "%saugustus_output" % self.main_out,
-    #                   "-zcf", "%saugustus_output/gffs.tar.gz" %
-    #                   self.main_out, "gffs", "--remove-files"], "bash", shell=False)
-    #     # single_copy_busco_sequences
-    #     self._p_open(["tar", "-C", "%s" % self.main_out, "-zcf",
-    #                   "%ssingle_copy_busco_sequences.tar.gz" % self.main_out,
-    #                   "single_copy_busco_sequences", "--remove-files"], "bash", shell=False)
+    def _run_bbtools(self):
+        self.bbtools_runner.configure_runner()
+        if self.bbtools_runner.check_previous_completed_run():
+            logger.info("Skipping BBTools run as already run")
+        else:
+            self.bbtools_runner.run()
+        if len(self.bbtools_runner.metrics) == 0:
+            self.bbtools_runner.parse_output()
 
-    # def set_rerun_busco_command(self, clargs):
-    #     """
-    #     This function sets the command line to call to reproduce this run
-    #     """
-    #     clargs.extend(["-sp", self._target_species])
-    #     super().set_rerun_busco_command(clargs)
+    def reset(self):
+        super().reset()
+        if self.bbtools_runner:
+            self.bbtools_runner.reset()
 
 
 class GenomeAnalysisProkaryotes(GenomeAnalysis):
@@ -122,15 +110,24 @@ class GenomeAnalysisProkaryotes(GenomeAnalysis):
         super().init_tools()
         self.prodigal_runner = ProdigalRunner()
 
+    def reset(self):
+        super().reset()
+        if (
+            self.prodigal_runner
+        ):  # If final run has already been run, then the prodigal_runner object in the final runner object will
+            # still be set to None
+            self.prodigal_runner.reset()
+
     @log("***** Run Prodigal on input to predict and extract genes *****", logger)
     def _run_prodigal(self):
         """
         Run Prodigal on input file to detect genes.
         :return:
         """
+        self.prodigal_runner.configure_runner()
         if self.restart and self.prodigal_runner.check_previous_completed_run():
             logger.info("Skipping Prodigal run as it has already completed")
-            self.prodigal_runner.get_gene_details()
+            self.prodigal_runner.run(restart=self.restart)
         else:
             self.restart = False
             self.config.set("busco_run", "restart", str(self.restart))
@@ -169,19 +166,13 @@ class GenomeAnalysisEukaryotes(GenomeAnalysis):
 
         return
 
+    def reset(self):
+        super().reset()
+
     @abstractmethod
     def run_analysis(self):
         super().run_analysis()
 
-    # def set_rerun_busco_command(self, clargs):
-    #     """
-    #     This function sets the command line to call to reproduce this run
-    #     """
-    #     clargs.extend(["-sp", self._target_species])
-    #     if self._augustus_parameters:
-    #         clargs.extend(["--augustus_parameters", "\"%s\"" % self._augustus_parameters])
-    #     super().set_rerun_busco_command(clargs)
-
 
 class GenomeAnalysisEukaryotesAugustus(BLASTAnalysis, GenomeAnalysisEukaryotes):
     def __init__(self):
@@ -220,13 +211,22 @@ class GenomeAnalysisEukaryotesAugustus(BLASTAnalysis, GenomeAnalysisEukaryotes):
         if self._long:
             self.optimize_augustus_runner = OptimizeAugustusRunner()
 
+    def reset(self):
+        super().reset()
+        self.augustus_runner.reset()
+        self.gff2gb_runner.reset()
+        self.new_species_runner.reset()
+        self.etraining_runner.reset()
+        if self._long:
+            self.optimize_augustus_runner.reset()
+
     def cleanup(self):
         try:
             self.augustus_runner.move_retraining_parameters()
             self.config.set(
                 "busco_run", "augustus_species", self._target_species_initial
             )  # Reset parameter for batch mode
-        except OSError:
+        except (OSError, AttributeError):
             pass
         super().cleanup()
 
@@ -292,11 +292,6 @@ class GenomeAnalysisEukaryotesAugustus(BLASTAnalysis, GenomeAnalysisEukaryotes):
         except NoGenesError:
             logger.warning("No genes found on Augustus rerun.")
 
-        # if self._tarzip:  # todo: zip folders with a lot of output
-        #     self._run_tarzip_augustus_output()
-        #     self._run_tarzip_hmmer_output()
-        # remove the checkpoint, run is done
-        # self._set_checkpoint()
         return
 
     @log("Running Augustus gene predictor on BLAST search results.", logger)
@@ -349,6 +344,7 @@ class GenomeAnalysisEukaryotesAugustus(BLASTAnalysis, GenomeAnalysisEukaryotes):
     )
     def _run_new_species(self):
         """Create new species config file from template"""
+        self.new_species_runner.configure_runner()
         if self.restart and self.new_species_runner.check_previous_completed_run():
             logger.info("Skipping new species creation as it has already been done")
         else:
@@ -382,6 +378,10 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
 
         self.metaeuk_runner = MetaeukRunner()
 
+    def reset(self):
+        super().reset()
+        self.metaeuk_runner.reset()
+
     def run_analysis(self):
         """This function calls all needed steps for running the analysis."""
         super().run_analysis()
@@ -391,6 +391,7 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
                 self._run_metaeuk(incomplete_buscos)
                 self.gene_details.update(self.metaeuk_runner.gene_details)
                 self.sequences_aa.update(self.metaeuk_runner.sequences_aa)
+                self.sequences_nt.update(self.metaeuk_runner.sequences_nt)
                 self.run_hmmer(
                     self.metaeuk_runner.pred_protein_seqs_modified,
                     busco_ids=incomplete_buscos,
@@ -404,9 +405,7 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
                 if i == 1:
                     logger.info("Metaeuk rerun did not find any genes")
                 else:
-                    raise BuscoError(
-                        "Metaeuk did not find any genes in the input file."
-                    )
+                    raise NoGenesError("Metaeuk")
 
         try:
             self.metaeuk_runner.combine_run_results()
@@ -417,7 +416,28 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
             self.metaeuk_runner.combined_pred_protein_seqs = (
                 self.metaeuk_runner.pred_protein_mod_files[0]
             )
-        self.hmmer_runner.write_buscos_to_file(self.sequences_aa)
+            self.metaeuk_runner.combined_nucleotides_seqs = (
+                self.metaeuk_runner.codon_mod_files[0]
+            )
+        self.hmmer_runner.write_buscos_to_file(self.sequences_aa, self.sequences_nt)
+        self.write_gff_files()
+
+    def write_gff_files(self):
+        busco_seqs_folders = [
+            self.hmmer_runner.single_copy_sequences_folder,
+            self.hmmer_runner.multi_copy_sequences_folder,
+            self.hmmer_runner.fragmented_sequences_folder,
+        ]
+        for folder in busco_seqs_folders:
+            existing_gff_files = glob.glob(os.path.join(folder, "*.gff"))
+            for f in existing_gff_files:
+                os.remove(f)
+
+        for gff_file in self.metaeuk_runner.gff_files[
+            ::-1
+        ]:  # Write GFF results to file, starting with the rerun results and then using the initial results
+            self.metaeuk_runner.gff_file = gff_file
+            self.metaeuk_runner.write_gff_files(*busco_seqs_folders)
 
     def _run_metaeuk(self, incomplete_buscos):
         self.metaeuk_runner.configure_runner(incomplete_buscos)
@@ -428,7 +448,7 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
             self.config.set("busco_run", "restart", str(self.restart))
             self.metaeuk_runner.run()
 
-        self.metaeuk_runner.edit_protein_file()
+        self.metaeuk_runner.edit_result_headers()
         self.metaeuk_runner.get_gene_details()  # The gene details contain the overlaps that were removed when editing
         # the protein file, but it doesn't matter, as it is just a look-up
         # dictionary
@@ -495,27 +515,34 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
                     )
                     else "1"
                 )
-
-                if run_found == "2":
-                    matches = subprocess.check_output(
-                        [
-                            "grep",
-                            "{}|{}|.*|{}|{}|".format(
-                                sequence, strand, gene_start, gene_end
-                            ),
-                            rerun_results,
-                        ]
-                    ).decode("utf-8")
-                else:
-                    matches = subprocess.check_output(
-                        [
-                            "grep",
-                            "{}|{}|.*|{}|{}|".format(
-                                sequence, strand, gene_start, gene_end
-                            ),
-                            initial_run_results,
-                        ]
-                    ).decode("utf-8")
+                try:
+                    if run_found == "2":
+                        matches = subprocess.check_output(
+                            [
+                                "grep",
+                                "{}|{}|.*|{}|{}|".format(
+                                    sequence, strand, gene_start, gene_end
+                                ),
+                                rerun_results,
+                            ]
+                        ).decode("utf-8")
+                    else:
+                        matches = subprocess.check_output(
+                            [
+                                "grep",
+                                "{}|{}|.*|{}|{}|".format(
+                                    sequence, strand, gene_start, gene_end
+                                ),
+                                initial_run_results,
+                            ]
+                        ).decode("utf-8")
+                except subprocess.CalledProcessError:
+                    raise BuscoError(
+                        "Unable to parse metaeuk results. This typically occurs because sequence headers "
+                        "contain pipes ('|'). Metaeuk uses pipes as delimiters in the results files. The "
+                        "additional pipes interfere with BUSCO's ability to accurately parse the results."
+                        "To fix this problem remove any pipes from sequence headers and try again."
+                    )
 
                 # The following line is a relic from when the previous grep search tried to match busco_id instead of
                 # gene details. The find_match method is still needed though to clean up the match, even though it
@@ -560,7 +587,6 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
                 for idx, row in busco_gene_group.iterrows():
                     low_coord = row["Start"]
                     high_coord = row["Stop"]
-                    score = row["Score"]
                     seq = row["Sequence"]
                     if min_coord:
                         min_coord = min(min_coord, low_coord)
@@ -584,6 +610,9 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
                 self.sequences_aa[new_gene_match] = self.metaeuk_runner.sequences_aa[
                     gene_match
                 ]
+                self.sequences_nt[new_gene_match] = self.metaeuk_runner.sequences_nt[
+                    gene_match
+                ]
         return hmmer_result_dict_new, matched_genes_new
 
     def exons_to_df(self, records):
@@ -648,8 +677,8 @@ class GenomeAnalysisEukaryotesMetaeuk(GenomeAnalysisEukaryotes):
         hmmer_match_details2 = self.hmmer_runner.parse_hmmer_output(
             hmmer_result2, busco_match2
         )
-        gene_id1 = list(hmmer_match_details1.keys())[0]
-        gene_id2 = list(hmmer_match_details2.keys())[0]
+        gene_id1 = match1["Orig gene ID"]
+        gene_id2 = match2["Orig gene ID"]
         if (
             hmmer_match_details1[gene_id1]["score"]
             > hmmer_match_details2[gene_id2]["score"]
