@@ -15,7 +15,7 @@ class ToolException(Exception):
     Module-specific exception
     """
 
-    def __init__(self, value):
+    def __init__(self, value=""):
         self.value = value
 
     def __str__(self):
@@ -31,6 +31,7 @@ class BaseRunner(Tool, metaclass=ABCMeta):
         super().__init__()
         self.run_number = 0
         self.input_file = self.config.get("busco_run", "in")
+        # type(self).summary["versions"]["busco"] = busco.__version__
         self.main_out = self.config.get("busco_run", "main_out")
         self.working_dir = (
             os.path.join(self.main_out, "auto_lineage")
@@ -44,11 +45,14 @@ class BaseRunner(Tool, metaclass=ABCMeta):
         self.lineage_dataset = self.config.get("busco_run", "lineage_dataset")
         self.domain = self.config.get("busco_run", "domain")
 
-        if not self.check_tool_available():
+        try:
+            self.check_tool_available()
+        except ToolException:
             raise ToolException(
                 "{} tool cannot be found. Please check the 'path' and 'command' parameters "
-                "provided in the config file. Do not include the command in the "
-                "path!".format(self.name)
+                "provided in the config file or make sure the tool is available in your working environment.".format(
+                    self.name
+                )
             )
         self.version = self.get_version()
         type(self).tool_versions[self.name] = self.version
@@ -64,43 +68,56 @@ class BaseRunner(Tool, metaclass=ABCMeta):
         self.logfile_path_err = (
             self.logfile_path_out.rpartition("_out.log")[0] + "_err.log"
         )
+        self.add_args = {}
 
     def init_checkpoint_file(self):
         self.checkpoint_file = os.path.join(self.output_folder, ".checkpoint")
 
-    def write_checkpoint_file(self):
+    def write_checkpoint_file(self, additional_args=[]):
         with open(self.checkpoint_file, "a") as cpt_file:
             cpt_file.write("Tool: {}\n".format(self.name))
             cpt_file.write("Version: {}\n".format(self.version))
             cpt_file.write("Run: {}\n".format(self.run_number))
-            cpt_file.write("Time: {}\n".format(time.strftime("%m/%d/%Y %H:%M:%S")))
+            for args in additional_args:
+                cpt_file.write("{}: {}\n".format(args[0], args[1]))
+            cpt_file.write("Time: {}\n".format(time.strftime("%d/%m/%Y %H:%M:%S")))
             cpt_file.write("Completed {} jobs\n\n".format(self.total))
 
-    def check_previous_completed_run(self):
+    def check_previous_completed_run(self, additional_args=[]):
         if not os.path.exists(self.checkpoint_file):
             return False
         else:
             with open(self.checkpoint_file, "r") as cpt_file:
+                block_size = 6 + len(additional_args)
                 lines = cpt_file.readlines()
-                tool_names = [s.strip().split(": ")[1] for s in lines[0::6]]
-                tool_versions = [s.strip().split(": ")[1] for s in lines[1::6]]
-                tool_run_numbers = [s.strip().split(": ")[1] for s in lines[2::6]]
+                tool_names = [s.strip().split(": ")[1] for s in lines[0::block_size]]
+                tool_versions = [s.strip().split(": ")[1] for s in lines[1::block_size]]
+                tool_run_numbers = [
+                    s.strip().split(": ")[1] for s in lines[2::block_size]
+                ]
+                self.add_args = {}
+                for a, arg in enumerate(additional_args):
+                    self.add_args[arg] = [
+                        s.strip().split(": ")[1]
+                        for s in lines[a + 3 :: block_size]
+                        if s.strip().split(": ")[0] == arg
+                    ]
                 try:
                     start_search = 0
                     while True:
                         tool_ind = tool_names.index(self.name, start_search)
-                        if str(self.version) != str(tool_versions[tool_ind]):
-                            logger.warning(
-                                "A previous run used {} version {}. "
-                                "The restarted run is using {} version "
-                                "{}".format(
-                                    self.name,
-                                    tool_versions[tool_ind],
-                                    self.name,
-                                    self.version,
-                                )
-                            )
                         if int(tool_run_numbers[tool_ind]) == int(self.run_number):
+                            if str(self.version) != str(tool_versions[tool_ind]):
+                                logger.warning(
+                                    "A previous run used {} version {}. "
+                                    "The restarted run is using {} version "
+                                    "{}".format(
+                                        self.name,
+                                        tool_versions[tool_ind],
+                                        self.name,
+                                        self.version,
+                                    )
+                                )
                             return True
                         elif int(tool_run_numbers[tool_ind]) < int(self.run_number):
                             start_search = tool_ind + 1
@@ -132,6 +149,10 @@ class BaseRunner(Tool, metaclass=ABCMeta):
     def configure_job(self, *args):
         pass
 
+    @abstractmethod
+    def configure_runner(self, *args):
+        self.init_checkpoint_file()
+
     @abstractmethod
     def generate_job_args(self):
         pass
@@ -179,14 +200,17 @@ class BaseRunner(Tool, metaclass=ABCMeta):
         try:
             self.get_tool_from_config()
         except ToolException:
-            self.get_tool_from_environment()
+            try:
+                self.get_tool_from_environment()
+            except ToolException:
+                raise
 
         return which(self.cmd) is not None  # True if tool available
 
     def get_tool_from_environment(self):
         which_tool = which(self.cmd)
         if not which_tool:
-            raise ToolException("Tool {} not found".format(self.name))
+            raise ToolException()
 
     def get_tool_from_config(self):
         """
@@ -197,20 +221,12 @@ class BaseRunner(Tool, metaclass=ABCMeta):
         :return:
         """
         if not self.config.has_section(self.name):
-            raise ToolException(
-                "Section for the tool [{}] is not present in the config file".format(
-                    self.name
-                )
-            )
+            raise ToolException()
 
         if not self.config.has_option(self.name, "path") or not self.config.get(
             self.name, "path"
         ):
-            raise ToolException(
-                "Key 'path' in the section [{}] is not present in the config file".format(
-                    self.name
-                )
-            )
+            raise ToolException()
 
         if self.config.has_option(self.name, "command"):
             executable = self.config.get(self.name, "command")
@@ -225,6 +241,11 @@ class BaseRunner(Tool, metaclass=ABCMeta):
     def get_version(self):
         return
 
+    @classmethod
+    def reset(cls):
+        BaseRunner.config = None
+        BaseRunner.tool_versions = {}
+
 
 class NoGenesError(Exception):
     def __init__(self, gene_predictor):
