@@ -23,9 +23,11 @@ class MKBLASTRunner(BaseRunner):
         self.output_db = os.path.join(self.db_path, os.path.basename(self.input_file))
         self.create_dirs(self.db_path)
         self.total = 1
-        self.init_checkpoint_file()
         self.run_number += 1
 
+    def configure_runner(self, *args):
+        super().configure_runner(args)
+
     @log("Creating BLAST database with input file", logger)
     def configure_job(self, *args):
         mkblast_job = self.create_job()
@@ -44,6 +46,9 @@ class MKBLASTRunner(BaseRunner):
 
         self.run_jobs()
 
+    def reset(self):
+        super().reset()
+
     def generate_job_args(self):
         yield
 
@@ -84,11 +89,23 @@ class TBLASTNRunner(BaseRunner):
         self.region_limit = self.config.getint("busco_run", "limit")
         self.flank = self._define_flank()
 
-        self.init_checkpoint_file()
+        self.blast_db = None
+        self.missing_and_frag_only = None
+        self.ancestral_variants = None
+        self.incomplete_buscos = None
+
+        self.ancestral_sfx = None
+        self.ancestral_file = None
+        self.query_file = None
+        self.output_suffix = None
+        self.rerun_query_file = None
+        self.blast_filename = None
+        self.coords_filename = None
 
     def configure_runner(
         self, blast_db, missing_and_frag_only, ancestral_variants, incomplete_buscos
     ):
+        super().configure_runner()
         self.run_number += 1
         self.blast_db = blast_db
         self.missing_and_frag_only = missing_and_frag_only
@@ -140,10 +157,6 @@ class TBLASTNRunner(BaseRunner):
         return self._output_folder
 
     def _define_flank(self):
-        """
-        TODO: Add docstring
-        :return:
-        """
         try:
             size = os.path.getsize(self.input_file) / 1000  # size in mb
             flank = int(size / 50)  # proportional flank size
@@ -163,6 +176,9 @@ class TBLASTNRunner(BaseRunner):
         self._check_output()
         return
 
+    def reset(self):
+        super().reset()
+
     def check_tool_dependencies(self):
         if (
             ".".join(self.version.split(".")[:-1]) not in ["2.2", "2.3"]
