@@ -1,15 +1,16 @@
 from busco.busco_tools.base import BaseRunner
 import os
 from collections import defaultdict
-import busco
 from busco.BuscoLogger import BuscoLogger
 from busco.BuscoLogger import LogDecorator as log
-from busco.BuscoConfig import BuscoConfig, BuscoConfigMain
+from busco.BuscoConfig import BaseConfig
 from Bio import SeqIO
 import csv
 import subprocess
 from busco.BuscoConfig import BuscoConfigAuto
 from busco.Exceptions import BatchFatalError, BuscoError
+import pandas as pd
+import busco
 
 logger = BuscoLogger.get_logger(__name__)
 
@@ -37,7 +38,7 @@ class HMMERRunner(BaseRunner):
         self.fragmented_sequences_folder = os.path.join(
             self.run_folder, "busco_sequences", "fragmented_busco_sequences"
         )
-        self.short_summary_file = os.path.join(self.run_folder, "short_summary.txt")
+
         self.cutoff_dict = {}
         self.single_copy_buscos = {}
         self.multi_copy_buscos = {}
@@ -91,10 +92,11 @@ class HMMERRunner(BaseRunner):
         self.missing_percent = 0
 
         self.hmmer_results_lines = None
-
-        self.init_checkpoint_file()
+        self._already_used_genes = None
+        self.missing_buscos = None
 
     def configure_runner(self, input_sequences, busco_ids, mode, gene_details):
+        super().configure_runner()
         self.run_number += 1
         self.input_sequences = input_sequences
         self.busco_ids = busco_ids
@@ -225,10 +227,10 @@ class HMMERRunner(BaseRunner):
         :raises BatchFatalError: if a Tool version is not supported
         """
         # check hmm version
-        if not self.version >= BuscoConfig.HMMER_VERSION:
+        if not self.version >= BaseConfig.HMMER_VERSION:
             raise BatchFatalError(
                 "HMMer version detected is not supported, please use HMMer v.{} +".format(
-                    BuscoConfig.HMMER_VERSION
+                    BaseConfig.HMMER_VERSION
                 )
             )
         return
@@ -306,7 +308,6 @@ class HMMERRunner(BaseRunner):
 
                         # Store bitscore matches for each gene match. If match below cutoff, discard.
                         if bit_score < float(self.cutoff_dict[busco_query]["score"]):
-                            # todo: introduce upper bound - consult to see what a reasonable value would be
                             continue
                         if gene_id not in records:
                             records[gene_id] = {
@@ -324,18 +325,19 @@ class HMMERRunner(BaseRunner):
                         records[gene_id]["env_coords"].append((env_start, env_end))
 
                     except IndexError as e:
-                        raise BuscoError(
-                            e, "Cannot parse HMMER output file {}".format(filename)
+                        logger.error(
+                            "Cannot parse HMMER output file {}".format(filename)
                         )
+                        raise BuscoError(e)
         return records
 
     def _sort_matches(self, matched_record, busco_query):
         """
         The HMMER gene matches are sorted into "complete", "v_large" and "fragmented" matches based on a comparison
         with the cutoff value specified in the dataset cutoff_scores file
-        :param matched_lengths: dict of (gene_id, total_matched_length) pairs
+        :param matched_record: dict of (gene_id, total_matched_length) pairs
         :param busco_query: BUSCO identifier
-        :type matched_lengths: dict
+        :type matched_record: dict
         :type busco_query: str
         :return: busco_complete, busco_vlarge, busco_fragment - three dictionaries of the form
         {gene_id: [{"bitscore": float, "length": int}, {...}, ...], ...}
@@ -390,19 +392,12 @@ class HMMERRunner(BaseRunner):
                 ]
             )
         elif self.run_number == 2:
-            hmmer_initial_run_files = [
-                os.path.join(self.initial_results_dir, f)
-                for f in os.listdir(self.initial_results_dir)
-                if not f.startswith(".")
-            ]
             hmmer_rerun_files = [
                 os.path.join(self.rerun_results_dir, f)
                 for f in os.listdir(self.rerun_results_dir)
                 if not f.startswith(".")
             ]
-            hmmer_results_files = sorted(
-                hmmer_rerun_files
-            )  # sorted(hmmer_initial_run_files + hmmer_rerun_files)
+            hmmer_results_files = sorted(hmmer_rerun_files)
         else:
             raise ValueError(
                 "HMMER should not be run more than twice in the same Run instance."
@@ -411,8 +406,9 @@ class HMMERRunner(BaseRunner):
         for filename in hmmer_results_files:
             busco_query = str(os.path.basename(filename).split(".")[0])
             matched_record = self.parse_hmmer_output(filename, busco_query)
+            filtered_records = self.remove_overlaps(matched_record)
             busco_complete, busco_vlarge, busco_fragment = self._sort_matches(
-                matched_record, busco_query
+                filtered_records, busco_query
             )
 
             # Add all information for this busco_id to the full dictionary
@@ -425,6 +421,74 @@ class HMMERRunner(BaseRunner):
 
         return
 
+    @staticmethod
+    def remove_overlaps(matched_records):
+        seq_ids = []
+        start_coords = []
+        end_coords = []
+        scores = []
+        try:
+            for record in matched_records:
+                seq_id, coords = record.split(":")
+                start_coord, end_coord = coords.split("-")
+                seq_ids.append(seq_id)
+                start_coords.append(start_coord)
+                end_coords.append(end_coord)
+                scores.append(matched_records[record]["score"])
+        except ValueError:  # for protein sequences there is no ":<coords>" suffix, so skip the overlap filtering
+            return matched_records
+
+        records_df = pd.DataFrame(
+            {
+                "Sequence": seq_ids,
+                "Start": start_coords,
+                "Stop": end_coords,
+                "Score": scores,
+            }
+        )
+        results_grouped = records_df.groupby("Sequence")
+        entries_to_remove = []
+        seq_ids = results_grouped.groups.keys()
+        for seq in seq_ids:
+            g1 = results_grouped.get_group(seq)
+            g1_sorted = g1.sort_values(
+                "Start"
+            )  # sort to facilitate a single-pass coordinate check
+            for idx1, row1 in g1_sorted.iterrows():
+                current_record = g1_sorted.loc[idx1]
+                start_val = current_record["Start"]
+                stop_val = current_record["Stop"]
+                current_seqid = "{}:{}-{}".format(
+                    current_record["Sequence"], start_val, stop_val
+                )
+                if (
+                    current_seqid in entries_to_remove
+                ):  # overlaps with removed entries don't count
+                    continue
+
+                matches = g1_sorted[g1_sorted["Start"] > start_val].loc[
+                    g1_sorted["Start"] < stop_val
+                ]  # find entries with a start coordinate between the current exon start and end
+                for idx2 in matches.index.values:
+                    if g1_sorted.loc[idx1]["Score"] >= g1_sorted.loc[idx2]["Score"]:
+                        ind_to_remove = idx2
+                    else:
+                        ind_to_remove = idx1
+                    record_to_remove = g1_sorted.loc[ind_to_remove]
+                    entries_to_remove.append(
+                        "{}:{}-{}".format(
+                            record_to_remove["Sequence"],
+                            record_to_remove["Start"],
+                            record_to_remove["Stop"],
+                        )
+                    )
+
+        filtered_records = {
+            i: matched_records[i] for i in matched_records if i not in entries_to_remove
+        }
+
+        return filtered_records
+
     def _update_used_gene_set(self, busco_dict):
         """
         Update set of already used genes to prevent processing the same gene twice.
@@ -811,7 +875,8 @@ class HMMERRunner(BaseRunner):
                             self.cutoff_dict[taxid]["sigma"] = 1
                         self.cutoff_dict[taxid]["length"] = length
                     except IndexError as e:
-                        raise BuscoError(e, "Error parsing the lengths_cutoff file.")
+                        logger.error("Error parsing the lengths_cutoff file.")
+                        raise BuscoError(e)
         except IOError:
             raise BuscoError(
                 "Impossible to read the lengths in {}".format(
@@ -836,7 +901,8 @@ class HMMERRunner(BaseRunner):
                         score = float(line[1])
                         self.cutoff_dict[taxid]["score"] = score
                     except IndexError as e:
-                        raise BuscoError(e, "Error parsing the scores_cutoff file.")
+                        logger.error("Error parsing the scores_cutoff file.")
+                        raise BuscoError(e)
         except IOError:
             raise BuscoError(
                 "Impossible to read the scores in {}".format(scores_cutoff_file)
@@ -888,15 +954,14 @@ class HMMERRunner(BaseRunner):
 
         with open(os.path.join(self.run_folder, "full_table.tsv"), "w") as f_out:
 
-            self._write_output_header(f_out)
+            self.write_output_header(f_out)
 
             with open(
                 os.path.join(self.run_folder, "missing_busco_list.tsv"), "w"
             ) as miss_out:
 
-                self._write_output_header(miss_out, missing_list=True)
+                self.write_output_header(miss_out, missing_list=True)
 
-                # todo: move to calculate busco percentages
                 missing_buscos_lines, missing_buscos = self._list_missing_buscos()
                 output_lines += missing_buscos_lines
 
@@ -914,19 +979,8 @@ class HMMERRunner(BaseRunner):
         return sorted_lines
 
     def produce_hmmer_summary(self):
-        self._get_busco_percentages()
 
         self.hmmer_results_lines.append("***** Results: *****\n\n")
-        self.one_line_summary_raw = "C:{}%[S:{}%,D:{}%],F:{}%,M:{}%,n:{}\t{}\n".format(
-            self.complete_percent,
-            self.s_percent,
-            self.d_percent,
-            self.f_percent,
-            self.missing_percent,
-            self.total_buscos,
-            "   ",
-        )
-        self.one_line_summary = "Results:\t{}".format(self.one_line_summary_raw)
         self.hmmer_results_lines.append(self.one_line_summary_raw)
         self.hmmer_results_lines.append(
             "{}\tComplete BUSCOs (C)\t\t\t{}\n".format(
@@ -956,7 +1010,9 @@ class HMMERRunner(BaseRunner):
             )
         )
         self.hmmer_results_lines.append(
-            "{}\tTotal BUSCO groups searched\t\t{}\n".format(self.total_buscos, "   ")
+            "{}\tTotal BUSCO groups searched\t\t{}\n".format(
+                self.dataset_nb_buscos, "   "
+            )
         )
 
         if isinstance(self.config, BuscoConfigAuto):
@@ -967,47 +1023,48 @@ class HMMERRunner(BaseRunner):
         else:
             self._log_one_line_hmmer_summary()
 
-        with open(self.short_summary_file, "w") as summary_file:
-
-            self._write_output_header(summary_file, no_table_header=True)
-            summary_file.write(
-                "# Summarized benchmarking in BUSCO notation for file {}\n"
-                "# BUSCO was run in mode: {}\n".format(self.input_file, self.mode)
-            )
-            if self.mode == "genome":
-                if self.config.get("busco_run", "domain") in ["prokaryota", "viruses"]:
-                    gene_predictor = "prodigal"
-                elif self.config.getboolean("busco_run", "use_augustus"):
-                    gene_predictor = "augustus"
-                else:
-                    gene_predictor = "metaeuk"
-                summary_file.write("# Gene predictor used: {}\n".format(gene_predictor))
-            summary_file.write("\n")
-
-            for line in self.hmmer_results_lines:
-                summary_file.write("\t{}".format(line))
-
-            tool_versions_lines = self.report_tool_versions()
-            for line in tool_versions_lines:
-                summary_file.write(line + "\n")
-
-            if (
-                self.config.getboolean("busco_run", "auto-lineage")
-                and isinstance(self.config, BuscoConfigMain)
-                and hasattr(self.config, "placement_files")
-            ):
-                summary_file.write("\nPlacement file versions:\n")
-                for placement_file in self.config.placement_files:
-                    summary_file.write("\t{}\n".format(placement_file))
-
         return
 
-    def report_tool_versions(self):
-        lines = []
-        lines.append("\nDependencies and versions:")
-        for key, value in type(self).tool_versions.items():
-            lines.append("\t{}: {}".format(key, value))
-        return lines
+    def record_results(self):
+        self._get_busco_percentages()
+        self.one_line_summary_raw = "C:{}%[S:{}%,D:{}%],F:{}%,M:{}%,n:{}\t{}\n".format(
+            self.complete_percent,
+            self.s_percent,
+            self.d_percent,
+            self.f_percent,
+            self.missing_percent,
+            self.total_buscos,
+            "   ",
+        )
+        self.one_line_summary = "Results:\t{}".format(self.one_line_summary_raw)
+        # type(self).summary["results"][
+        #     "one_line_summary"
+        # ] = self.one_line_summary_raw.strip()
+        # type(self).summary["results"]["C"] = self.single_copy + self.multi_copy
+        # type(self).summary["results"]["S"] = self.single_copy
+        # type(self).summary["results"]["D"] = self.multi_copy
+        # type(self).summary["results"]["F"] = self.only_fragments
+        # type(self).summary["results"]["M"] = (
+        #     self.total_buscos - self.single_copy - self.multi_copy - self.only_fragments
+        # )
+        # type(self).summary["parameters"]["mode"] = self.mode
+        #
+        # if self.mode == "genome":
+        #     if self.config.get("busco_run", "domain") in ["prokaryota", "viruses"]:
+        #         gene_predictor = "prodigal"
+        #     elif self.config.getboolean("busco_run", "use_augustus"):
+        #         gene_predictor = "augustus"
+        #     else:
+        #         gene_predictor = "metaeuk"
+        #     type(self).summary["parameters"]["gene_predictor"] = gene_predictor
+        # type(self).summary["lineage_dataset"]["name"] = self.lineage_dataset
+        # type(self).summary["lineage_dataset"][
+        #     "creation_date"
+        # ] = self.dataset_creation_date
+        # type(self).summary["lineage_dataset"][
+        #     "number_species"
+        # ] = self.dataset_nb_species
+        # type(self).summary["lineage_dataset"]["total_buscos"] = self.dataset_nb_buscos
 
     @log("{}", logger, attr_name="hmmer_results_lines", apply="join", on_func_exit=True)
     def _produce_full_hmmer_summary(self):
@@ -1028,13 +1085,17 @@ class HMMERRunner(BaseRunner):
     def _log_one_line_hmmer_summary(self):
         return
 
-    def _write_output_header(
+    def write_output_header(
         self, file_object, missing_list=False, no_table_header=False
     ):
         """
         Write a standardized file header containing information on the BUSCO run.
         :param file_object: Opened file object ready for writing
         :type file_object: file
+        :param missing_list: Add list of missing BUSCOs
+        :type missing_list: bool
+        :param no_table_header: Include table header
+        :type no_table_header: bool
         :return:
         """
         file_object.write(
@@ -1048,8 +1109,6 @@ class HMMERRunner(BaseRunner):
                 self.dataset_nb_buscos,
             )
         )
-        # if isinstance(self._config, BuscoConfigMain):  # todo: wait until rerun command properly implemented again
-        #     file_object.write("# To reproduce this run: {}\n#\n".format(self._rerun_cmd))
 
         if no_table_header:
             pass
